void light_on(byte brightness)
{
  static unsigned long light_on_time;
  static long nightlength, night_time;
  static byte lamp_on = 0;
  static byte night_run = 0;
  char day;
  
  day = daylight();

  if (lamp_on == 0)
  {
    if (day == 'N' && night_run == 0)
    {
      night_run = 1;
      lamp_on = 1;
      light_on_time = millis();
      nightlength = night_length() / 2;
    }
    else if (day == 'Y')
    {
      night_run = 0;
    }
  }
  else
  { 
    if ((millis() - light_on_time) > nightlength)
    {
      lamp_on = 0;
    }
  }
       
  
  if (lamp_on == 1) analogWrite(number_light, brightness);
  else analogWrite(number_light, 0);
  
  Serial.println(night_run, DEC);
}



int light_level() // return % of light level 300ohms for daylight to ??? in darkness with R1=10k resistor
{
  int d = analogRead(light_sensor);
  
  int c = map(d, 70, 1023, 0, 100); 
  
  return constrain(c, 0, 100); // Prevent returning outside of 0-100%
}

char daylight()
{
  #define crossover 78 // level to switch
  #define hist 10 // histerisis amount
  
  static char day = 'Y';
  int x = light_level();
  
  switch (day)
  {
    case 'Y':
      if (x <= crossover) day = 'N';
      break;
    case 'N':
      if (x > (crossover + hist)) day = 'Y';
      break;
  }
  
  return day;
}

long night_length()
{
  static unsigned long darktime = 43200000; // preset 12 hours darkness
  static unsigned long sunset;
  static unsigned long sunrise;
  static char last_light = 'Y';
  
  char current_light = daylight();
  
  Serial.println(current_light, BYTE);
  
  if (current_light != last_light)
  {
    if (current_light == 'Y' && last_light == 'N')
    {
      sunrise = millis();
      last_light = current_light;
      
      if (millis_overrun() == 1)
      {
        darktime = 4294967295 - sunset + sunrise;
      }
      else
      {
        darktime = sunrise - sunset;
      }
    }
    else if (current_light == 'N' && last_light == 'Y')
    {
      sunset = millis();
      last_light = current_light;
    }
  }
  
  return darktime;
}

byte millis_overrun()
{
  static unsigned long old_millis = 0;
  unsigned long new_millis = millis();
  
  if (old_millis > new_millis)
  {
    old_millis = new_millis;
    return 1;
  }
  else
  {
    old_millis = new_millis;
    return 0;
  }
}
